@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -22,7 +22,7 @@
   @file    actions.hpp
   @brief   Implements base class Task, TaskFactory and the various supported
            actions (derived from Task).
-  @version $Rev: 1692 $
+  @version $Rev: 1760 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Dec-03, ahu: created
@@ -184,6 +184,16 @@ namespace Action {
         int printTag(const Exiv2::ExifData& exifData,
                      const std::string& key,
                      const std::string& label ="") const;
+        //! Type for an Exiv2 Easy access function
+        typedef Exiv2::ExifData::const_iterator (*EasyAccessFct)(const Exiv2::ExifData& ed);
+        /*!
+          @brief Print one summary line with a label (if provided) and requested
+                 data. A line break is printed only if a label is provided.
+          @return 1 if a line was written, 0 if the information was not found.
+         */
+        int printTag(const Exiv2::ExifData& exifData,
+                     EasyAccessFct easyAccessFct,
+                     const std::string& label) const;
 
     private:
         virtual Print* clone_() const;
